<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Cron;

use Exception;
use FiloBlu\RuleDayFromBirthday\Helper\Data as Helper;
use FiloBlu\RuleDayFromBirthday\Controller\Adminhtml\Promo\Quote\Birthdaycoupon;
use FiloBlu\RuleDayFromBirthday\Helper\TransformerHelper;
use Magento\Framework\Event\ManagerInterface;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Stdlib\DateTime\DateTime;

/**
 * @class SendBulkSubscriber
 * @package FiloBlu\BirthdayCoupon\Cron
 */
class SendBulkSubscriber extends SendBulkAbstract
{

    /** @var string  */
    const TYPE = 'subscriber';

    /**
     * @var TransformerHelper
     */
    private $transformerHelper;

    /**
     * @param Helper $helper
     * @param Birthdaycoupon $birthdaycouponGenerator
     * @param ManagerInterface $eventManager
     * @param DateTime $dateTime
     * @param TransformerHelper $transformerHelper
     */
    public function __construct(
        Helper            $helper,
        Birthdaycoupon    $birthdaycouponGenerator,
        ManagerInterface  $eventManager,
        DateTime          $dateTime,
        TransformerHelper $transformerHelper
    ) {
        $this->transformerHelper = $transformerHelper;
        parent::__construct($helper, $birthdaycouponGenerator, $eventManager, $dateTime);
    }

    /**
     * @throws LocalizedException
     * @throws InputException
     * @throws Exception
     */
    public function sendBulkSubscriber(): SendBulkSubscriber
    {
        $birthdayDate = $this->getBirthdayDate();
        $subscribers = $this->helper->getSubscribersForDate($birthdayDate);
        if (!$subscribers) {
            return $this;
        }

        $contacts = $this->transformerHelper->transformSubscribers($subscribers);
        parent::sendBulk($contacts, self::TYPE);

        return $this;
    }
}
